import re
import os
from django.template import Library, Node, TemplateSyntaxError
from ..jstools.conf import settings
from ..jstools.helpers import url_to_path
from os.path import isfile


register = Library()
string_re = re.compile(r'^(\'|")(?P<string>.+)\1$')


class ScriptsNode(Node):
    def __init__(self, nodelist, build):
        self.nodelist = nodelist
        self.build = build

    def __repr__(self):
        return "<ScriptsNode>"

    def render(self, context):
        if not settings.DEBUG:
            path = url_to_path(self.build)
            if isfile(path):
                t = int(os.path.getmtime(path))
                return '<script src="%s%s?%s"></script>' % (settings.STATIC_URL,
                                                            self.build, t)
        block = self.nodelist.render(context)
        urls = block.replace('\r\n', '\n').split('\n')
        scripts = []
        for url in urls:
            url = url.strip()
            if url:
                if not url.startswith('http://') and not url.startswith('/'):
                    url = settings.STATIC_URL + url
                scripts.append('<script src="%s"></script>' % url)
        return '\n'.join(scripts)


@register.tag
def scripts(parser, token):
    args = token.split_contents()
    if len(args) != 2:
        raise TemplateSyntaxError("Invalid syntax.")
    m = string_re.match(args[1])
    if not m:
        raise TemplateSyntaxError("First Argument must be a string.")
    build = m.group('string')
    nodelist = parser.parse(('endscripts',))
    parser.delete_first_token()
    return ScriptsNode(nodelist, build)




#coding=utf-8
from __future__ import with_statement
import os
import re
import urllib2
from django.core.exceptions import ImproperlyConfigured
from django.core.management.base import BaseCommand, CommandError
from django.core.urlresolvers import resolve
from django.http import HttpRequest
from django.template import Template, Context
from jstools.conf import settings
from jstools.helpers import url_to_path
from optparse import make_option
from os.path import abspath, dirname, join as pjoin, isdir, splitext
from subprocess import Popen, PIPE
from urlparse import urlparse
try:
    from django.utils.importlib import import_module
except:
    from jstools.importlib import import_module


scripts_re = re.compile(r'{%\s*scripts\s+(\'|")(?P<build>.+?)\1\s*%}(?P<scripts>.*?){%\s*endscripts\s*%}', re.S)


class JSToolsError(Exception):
    pass


def write_to_tmp(s, name):
    d = settings.JSTOOLS_TMPDIR
    if not isdir(d):
        os.makedirs(d)
    name = name.replace('/', u'⁄')
    tmp = pjoin(d, name)
    with open(tmp, 'w') as fp:
        fp.write(s)
    return tmp

def handle_extensions(extensions=('html',)):
    """
    organizes multiple extensions that are separated with commas or passed by
    using --extension/-e multiple times.

    for example: running 'django-admin makemessages -e js,txt -e xhtml -a'
    would result in a extension list: ['.js', '.txt', '.xhtml']

    >>> handle_extensions(['.html', 'html,js,py,py,py,.py', 'py,.py'])
    ['.html', '.js']
    >>> handle_extensions(['.html, txt,.tpl'])
    ['.html', '.tpl', '.txt']
    """
    ext_list = []
    for ext in extensions:
        ext_list.extend(ext.replace(' ','').split(','))
    for i, ext in enumerate(ext_list):
        if not ext.startswith('.'):
            ext_list[i] = '.%s' % ext_list[i]
    return set(ext_list)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--extension', '-e', dest='extensions',
            help=('The file extension(s) to examine (default: ".html", '
                  'separate multiple extensions with commas, or use -e '
                  'multiple times)'),
            action='append'),
        make_option('--compilation_level', '-l', dest='compilation_level',
                default='SIMPLE_OPTIMIZATIONS',
                help = ("Specifies the compilation level to use. "
                        "Options: WHITESPACE_ONLY, SIMPLE_OPTIMIZATIONS, "
                        "ADVANCED_OPTIMIZATIONS; default: SIMPLE_OPTIMIZATIONS")),
    )
    help = "Compiles javascript files defined in html documents."

    def handle(self, *args, **options):
        compilation_level = options['compilation_level']
        extensions = options.get('extensions') or ['html']
        extensions = handle_extensions(extensions)

        template_dirs = []
        def has_loader(*args):
            for loader in args:
                if loader in settings.TEMPLATE_LOADERS:
                    return True
            return False

        if has_loader('django.template.loaders.filesystem.Loader',
                      'django.template.loaders.filesystem.load_template_source'):
            for template_dir in settings.TEMPLATE_DIRS:
                template_dirs.append(template_dir)
        if has_loader('django.template.loaders.app_directories.Loader',
                      'django.template.loaders.app_directories.load_template_source'):
            for app in settings.INSTALLED_APPS:
                try:
                    mod = import_module(app)
                except ImportError, e:
                    raise ImproperlyConfigured, 'ImportError %s: %s' % (app, e.args[0])
                template_dir = abspath(pjoin(dirname(mod.__file__), 'templates'))
                template_dirs.append(template_dir)

        templates = []
        for template_dir in template_dirs:
            if isdir(template_dir):
                for (dirpath, dirnames, filenames) in os.walk(template_dir):
                    for f in filenames:
                        if splitext(f)[1] in extensions:
                            templates.append(pjoin(dirpath, f))

        builds = set()
        for template in templates:
            with open(template, 'r') as fp:
                data = fp.read()
            blocks = scripts_re.finditer(data)
            for block in blocks:
                build = block.group('build')
                if build in builds:
                    raise JSToolsError('Destination: %s has already been '
                                       'compiled to.')
                else:
                    builds.add(build)
                t = Template(block.group('scripts'))
                scripts = t.render(Context({})).strip()
                args = ['java', '-jar', settings.JSTOOLS_CLOSURE_COMPILER,
                        '--compilation_level', compilation_level]
                for url in scripts.replace('\r\n', '\n').split('\n'):
                    url = url.strip()
                    if url:
                        if url.startswith('http://'):
                            data = urllib2.urlopen(url).read()
                            path = write_to_tmp(data, url)
                        elif url.startswith('/'):
                            view, a, kw = resolve(urlparse(url)[2])
                            kw['request'] = HttpRequest()
                            response = view(*a, **kw)
                            path = write_to_tmp(response.content, url)
                        else:
                            path = url_to_path(url)
                        args.append('--js=%s' % path)
                args.append('--js_output_file=%s' % url_to_path(build))
                print 'Compiling scripts in `%s` to `%s`' % (template, build)
                p = Popen(args)
                p.wait()




import os
from os.path import join as pjoin, abspath, dirname
from django.conf import settings


JSTOOLS_CLOSURE_COMPILER = abspath(
    pjoin(dirname(__file__),
    os.pardir,
    'compiler',
    'compiler.jar'
    )
)

JSTOOLS_NAMESPACE = 'JSTOOLS'
JSTOOLS_TMPDIR = '/tmp/jstools/'

if hasattr(settings, 'STATIC_URL'):
    STATIC_URL = settings.STATIC_URL
else:
    STATIC_URL = settings.MEDIA_URL

if hasattr(settings, 'STATIC_ROOT'):
    STATIC_ROOT = settings.STATIC_ROOT
else:
    STATIC_ROOT = settings.MEDIA_ROOT




import re
from django.core.urlresolvers import get_resolver, get_script_prefix
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils import simplejson
from django.utils.safestring import mark_safe
from jstools.conf import settings


group_re = re.compile(r'%\([^)]+\)s')


def jshelper(request):
    resolver = get_resolver(None)
    prefix = get_script_prefix()
    urls = {}
    for name, L in resolver.reverse_dict.items():
        if isinstance(name, str):
            matches, pattern = L
            for result, params in matches:
                url = group_re.sub('%s', prefix + result)
                urls[name] = url
    return render_to_response('jstools/jshelper.js', {
        'settings': settings,
        'NS': settings.JSTOOLS_NAMESPACE,
        'urls': mark_safe(simplejson.dumps(urls)),
    }, RequestContext(request))

